#!/usr/bin/env python
# coding: UTF-8
import lang_en
import lang_de
import sys
import os, os.path
import subprocess as sub
import gobject
import re, string
import datetime, time
import signal
import urllib
import ConfigParser as config
from threading import Thread
try:
 	import pygtk
  	pygtk.require("2.6")
except:
  	pass
try:
	import gtk
  	import gtk.glade
except:
	sys.exit(1)
	
#gtk.gdk.threads_init()
gobject.threads_init()

basedir = os.path.dirname(os.path.realpath(__file__))
if not os.path.exists(os.path.join(basedir, "tragtor.py")):
    if os.path.exists(os.path.join(os.getcwd(), "tragtor.py")):
        basedir = os.getcwd()
sys.path.insert(0, basedir)

class tragtorGUI(Thread):
	"""This class is a GUI for ffmpeg"""
	
	_TITLE = "traGtor 0.5"
	_VERSION = "0.5.2"
	_AUTHOR = "Markus Schmidt in 2008\nhttp://mein-neues-blog.de"
	
	_LANGUAGE = "en"
	_LANGUAGES = ["en", "de"]
	
	_AUDIOSAMPLERATES = [8000,11025,22050,32000,44056,44100,47250,48000,50000,50400,88200,96000,176400,192000]
	_AUDIOBITRATES = [ 8, 16, 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320,
		384, 512, 1250, 1411, 5000, 15000, 36000, 54000]
	_DEFAULTBITRATE = 128
	_AUDIOCHANNELS = [ 1, 2, 3, 4, 5, 6, 7, 8 ]
	_SCALINGMETHODS = [ "Pad", "Crop", "Distort" ]
	_CACHEFORMAT = ["Width", "Height", "Framerate", "Bitrate", "AudioBitrate",
		"AudioChannels", "AudioSamplerate", "Offset", "Duration", "Ratio"]
	_PAGES = ["infile", "format", "meta", "outfile", "proceed", "settings"]
	_COMMAND = "ffmpeg -i '%s' %s '%s'"
	
	_ERRORHANDLING = "3"
	
	_CFG_PATH = os.path.expanduser("~") + "/.tragtor"
	_CFG_PRESETS = "presets.cfg"
	_CFG_SETTINGS = "tragtor.cfg"
	_DEFAULT_SETTINGS = {"language":"de"}
	_DEFAULT_PRESET = {"Width":320, "Height":240, "Ratio":1.3333333, "FixedRatio":True, "Bitrate":1024, "Framerate":25, "Deinterlace":False, "AudioSamplerate":44100, "AudioBitrate":128, "ForceCodec":"", "AudioChannels":2, "DisableAudio":False, "Codec":"", "DisableVideo":False, "Offset":0, "Duration":0, "Scaling":0, "PadColor":"#000000000000", "HiQ":True, "2Pass":False}
	
	_inFile = None
	_outFile = None
	_defFile = os.path.expanduser("~")
	_argInFile = False
	_argOutFile = False
	_inFileReadable = False
	_inFileAccessible = False
	_outFileWriteable = False
	_outFileAccessible = False
	_outFileSize = 0
	
	_canProceed = False
	_hasVideo = False
	_hasAudio = False
	
	_outFileFormat = []
	_inFileStreams = []
	_inFileFormat = []
	_inStream = 0
	
	_defaultWidth = 320
	_defaultHeight = 240
	_defaultRatio = 1.3333333
	_defaultFixedRatio = True
	_defaultBitrate = 1024
	_defaultFramerate = 25
	_defaultDeinterlace = False
	_defaultAudioSamplerate = 44100
	_defaultAudioBitrate = 128
	_defaultForceCodec = ""
	_defaultAudioChannels = 2
	_defaultDisableAudio = False
	_defaultCodec = ""
	_defaultDisableVideo = False
	_defaultOffset = 0
	_defaultDuration = 0
	_defaultScaling = 0
	_defaultPadColor = "#000000000000"
	_defaultHiQ = True
	_defaultBitrateVariation = 10
	_default2Pass = False
		
	_ffmpegVersion = ""
	_compileOptions = ""
	_openFileFormats = []
	_saveFileFormats = []
	_openAudioCodecs = []
	_openVideoCodecs = []
	_saveAudioCodecs = []
	_saveVideoCodecs = []
	
	_metaTitle = ""
	_metaAuthor = ""
	_metaCopyleft = ""
	_metaComment = ""
	_metaEnable = False
	_defaultMetaEnable = False
	
	_eec = 1
	_inProgress = False
	_cancelProcess = False
	_execute = ""
	_currentPass = 1
	_currpass = None
	
	_formatMaxHeight = 1080
	_formatMaxWidth = 1920
	
	_languageCombos = [
			["formatScaling", "format_scalingmethods_list"]
			]

	def __init__(self, argv=sys.argv):
		"""Initiate the program"""
		
		signal.signal(signal.SIGINT, signal.SIG_IGN)
		signal.signal(signal.SIGINT, self.ctrlC)
		
		### FIRST STEPS
		
		# change working directory for arguments
		#os.chroot(os.getcwd())
		
		# read ffmpeg's options and config
		self.ffmpegConfig = ffmpegConfig()

		#Thread.__init__(self)
		
		# create a window and it's corresponding internal objects
		self.createWindow()
		self.createWidgetObjects()
				
		# Create config file paths
		self._CFG_PRESETS = os.path.join(self._CFG_PATH, self._CFG_PRESETS)
		self._CFG_SETTINGS = os.path.join(self._CFG_PATH, self._CFG_SETTINGS)
		# Initiate config objects
		self.config = self.start_config(self._CFG_SETTINGS, self._DEFAULT_SETTINGS, ["Settings"], True)
		self.presets = self.start_config(self._CFG_PRESETS, self._DEFAULT_PRESET, [], True)
		
		# Set Language
		if self.config.get("Settings", "language"):
			self._LANGUAGE = self.config.get("Settings", "language")
		else:
			try:
				ln = gtk.get_default_language()
				i = self._LANGUAGES.index(ln.to_string()[0:2])
				self._LANGUAGE = self._LANGUAGES[i]
			except:
				True
		self.setLanguage()
		
		
		### CREATE MISSING WIDGETS AND GADGETS
		
		# inFileText Object
		self.inFileText = gtk.TextBuffer()
		self.inFileInfo.set_buffer(self.inFileText)
		
		# ffmpegOutText Object
		self.ffmpegOutText = gtk.TextBuffer()
		self.ffmpegOut.set_buffer(self.ffmpegOutText)
		
		# Language Selector
		for lan in self._LANGUAGES:
			self.languageSelector.append_text(lan)
		self.languageSelector.set_active(self._LANGUAGES.index(self._LANGUAGE))
		
		# Presets
		for pre in self.presets.sections():
			self.presetName.append_text(pre)
		
		#AudioSamplerates
		for lan in self._AUDIOSAMPLERATES:
			self.formatAudioSamplerate.append_text("%i" % lan)
		
		#AudioBitrates
		for lan in self._AUDIOBITRATES:
			self.formatAudioBitrate.append_text("%i" % lan)

		#AudioChannels
		for lan in self._AUDIOCHANNELS:
			self.formatAudioChannels.append_text("%i" % lan)
		
		#AudioCodecs
		self.formatForceCodec.append_text("")
		for lan in self.ffmpegConfig._saveAudioCodecs:
			self.formatForceCodec.append_text(lan)
		
		#VideoCodecs
		self.formatCodec.append_text("")
		for lan in self.ffmpegConfig._saveVideoCodecs:
			self.formatCodec.append_text(lan)
		
		#OutfileSuffix
		self.outFileSuffix.append_text(self.lang.output_file_suffix_auto)
		for lan in self.ffmpegConfig._saveFileFormats:
			self.outFileSuffix.append_text(lan[0])
		
		#in/outFile Filters
		self.reloadInFileFilter()
		self.reloadOutFileFilter()
		
		#duration-adjustment
		self.formatDurationAdjustment = gtk.Adjustment(0, 0, 21600, 0.001, 1, 0)
		self.formatDuration.set_adjustment(self.formatDurationAdjustment)
		
		#offset-adjustment
		self.formatOffsetAdjustment = gtk.Adjustment(0, 0, 21600, 0.001, 1, 0)
		self.formatOffset.set_adjustment(self.formatOffsetAdjustment)
		
		#hide changed-icons
		for i in self._CACHEFORMAT:
			if self.window.get_widget("button" + i):
				self.window.get_widget("button" + i).hide()
				
		#title
		self.window.get_widget("tragtor").set_title(self._TITLE + " :: " + eval("self.lang." + self._PAGES[0] + "_title"))
		
		# Arguments
		try:
			self._argInFile = sys.argv[1]
			if self._argInFile[:7] == "file://":
				self._argInFile = self._argInFile[8:]
			elif self._argInFile[0] != "/":
				self._argInFile = os.path.normpath(os.getcwd() + "/" + self._argInFile)
		except IndexError:
			try:
				presetfile = self.config.get("Settings", "inFile")
				if os.path.isfile(presetfile):
					self._argInFile = presetfile
			except:
				self._argInFile = False
			
		try:
			self._argOutFile = sys.argv[2]
			if self._argOutFile[0] != "/":
				self._argOutFile = os.path.normpath(os.getcwd() + "/" + self._argOutFile)
		except IndexError:
			self._argOutFile = False			
		
		### Communicate the rest
		
		self.connectWindow()
		self.reset()

#######################################
# PRESETS
#######################################	
	
	def deletePreset(self, widget):
		"""deletes a preset from config and combobox"""
		name = self.presetName.child.get_text()
		delete = False
		if self.presets.has_section(name):
			message = self.lang.question_delete_preset % name
			image = gtk.gdk.pixbuf_new_from_file(
				os.path.join(os.path.dirname(
					os.path.realpath(__file__)
				), "question.svg")
			)
			delete = self.messageDialog(
				message, self.lang.question_title,
				gtk.MESSAGE_WARNING,
				gtk.BUTTONS_OK_CANCEL,
				image)
		if delete == gtk.RESPONSE_OK:
			self.presets.remove_section(name)
			self.presets.write(open(self._CFG_PRESETS, "w"))
			#try:
			model = self.presetName.get_model()
			i = model.get_iter_first()
			while i:
				if model.get_value(i, 0) == name:
					sel = i
					break
				i = model.iter_next(i)
			try:
				model.remove(sel)
			except:
				1
			self.presetName.child.set_text("")
	
	def openPreset(self, widget):
		"""opens a preset from config and sets the values"""
		name = self.presetName.child.get_text()
		if self.presets.has_section(name):
			self.presetFormat(name)
			self.fillFormat()
		
	def savePreset(self, widget):
		"""saves a preset to config and adds it to combobox"""
		save = False
		name = self.presetName.child.get_text()
		if not self.presets.has_section(name) and name != "":
			self.presets.add_section(name);
			self.presetName.append_text(name)
			save = True
		elif name != "":
			message = self.lang.question_overwrite_preset % name
			image = gtk.gdk.pixbuf_new_from_file(
				os.path.join(os.path.dirname(
					os.path.realpath(__file__)
				), "question.svg")
			)
			save = self.messageDialog(
				message,
				self.lang.question_title,
				gtk.MESSAGE_WARNING,
				gtk.BUTTONS_OK_CANCEL,
				image)
		if save == gtk.RESPONSE_OK or save == True:
			for i in self._DEFAULT_PRESET:
				self.presets.set(name, i, eval("self._format" + i))
			self.presets.write(open(self._CFG_PRESETS, "w"))
			self.presets.readfp(open(self._CFG_PRESETS, "r"))

#######################################
# HANDLERS FOR GUI-INTERACTION
#######################################	

	def connectWindow(self):
		"""set all connections to the signals in the main-window"""
		handlers = {"destroy" : self.destroy,
				"on_notebook_change" : self.changeNotebook,
				"on_inFileChange" : self.changeInFile,
				"on_inFileDragDrop" : self.dragDropInFile,
				"on_outFileChange" : self.changeOutDirectory,
				"on_outFileSuffix_changed" : self.changeOutFileSuffix,
				"on_outFileName_change" : self.changeOutFileName,
				"on_metaToggle_clicked" : self.changeToggleMeta,
				"on_languageSelector_changed" : self.changeLanguage,
				"on_buttonProceed_clicked" : self.confirm,
				"on_buttonReload_clicked" : self.triggerReload,
				"on_buttonReset_clicked" : self.triggerReset,
				"on_labelSettingsCredits_release_event" : self.troubleShooter,
				"on_formatWidth_changed" : self.changeWidth,
				"on_formatHeight_changed" : self.changeHeight,
				"on_formatRatio_changed" : self.changeRatio,
				"on_formatFixedRatio_clicked" : self.changeFixedRatio,
				"on_formatFramerate_changed" : self.changeFramerate,
				"on_formatDeinterlace_clicked" : self.changeDeinterlace,
				"on_formatAudioSamplerate_changed" : self.changeAudioSamplerate,
				"on_formatAudioBitrate_changed" : self.changeAudioBitrate,
				"on_formatForceCodec_changed" : self.changeForceCodec,
				"on_formatAudioChannels_changed" : self.changeAudioChannels,
				"on_formatDisableAudio_clicked" : self.changeDisableAudio,
				"on_formatDisableVideo_clicked" : self.changeDisableVideo,
				"on_format2Pass_clicked" : self.change2Pass,
				"on_formatCodec_changed" : self.changeCodec,
				"on_formatBitrate_changed" : self.changeBitrate,
				"on_formatOffset_changed" : self.changeOffset,
				"on_formatDuration_changed" : self.changeDuration,
				"on_formatScaling_changed" : self.changeScaling,
				"on_formatPadColor_changed" : self.changePadColor,
				"on_buttonWidth_clicked" : self.resetFormatValue,
				"on_buttonHeight_clicked" : self.resetFormatValue,
				"on_buttonRatio_clicked" : self.resetFormatValue,
				"on_buttonBitrate_clicked" : self.resetFormatValue,
				"on_buttonFramerate_clicked" : self.resetFormatValue,
				"on_buttonAudioChannels_clicked" : self.resetFormatValue,
				"on_buttonAudioSamplerate_clicked" : self.resetFormatValue,
				"on_buttonAudioBitrate_clicked" : self.resetFormatValue,
				"on_buttonOffset_clicked" : self.resetFormatValue,
				"on_buttonDuration_clicked" : self.resetFormatValue,
				"on_presetSave_clicked" : self.savePreset,
				"on_presetDelete_clicked" : self.deletePreset,
				"on_presetOpen_clicked" : self.openPreset,
				}
				
		self.window.signal_autoconnect(handlers)	
	
	def resetFormatValue(self, widget):
		"""handler for reset-icons wich appear if a value differs from default"""
		objectName = widget.get_name()[6:]
		targetWidget = eval("self.format" + objectName)
		if targetWidget.class_path()[-16:] != "GtkComboBoxEntry":
			targetWidget.set_value(eval("self._inFile" + objectName))
		else:
			targetWidget.child.set_text("%i" % eval("self._inFile" + objectName))
		exec("self._format" + objectName + " = self._inFile" + objectName)
		self.setFormatChanged(objectName)
		
	
	def changeNotebook(self, widget, page, pagenum):
		"""triggers some updates of widgets and sets the title"""
		self.window.get_widget("tragtor").set_title(self._TITLE + " :: " + eval("self.lang." + self._PAGES[pagenum] + "_title"))
		self.analyzeOutFile()
		self.checkProceed()
		
	def changeWidth(self, widget):
		"""calculates height, ratio on change of width
		and sets the other values depending on fixedRatio-value"""
		widget.update()
		self.formatRatio.update()
		self.formatHeight.update()
		self._formatWidth = round(widget.get_value())
		if self._formatFixedRatio and self._formatWidth > 0 and self._formatRatio > 0:
			if self._formatWidth / self._formatRatio < self._formatMaxHeight:
				self._formatHeight = self._formatWidth / self._formatRatio
				self.formatHeight.set_value(round(self._formatHeight))
		else:
			self._formatRatio = self.roundFloat(self._formatWidth / self._formatHeight, 5)
			self.formatRatio.set_value(self._formatRatio)
		self.checkProceed()
		self.setFormatChanged("Width")
					
	def changeHeight(self, widget):
		"""calculates width, ratio on change of height
		and sets the other values depending on fixedRatio-value"""			
		widget.update()
		self.formatRatio.update()
		self.formatWidth.update()
		self._formatHeight = round(widget.get_value())
		if self._formatFixedRatio and self._formatHeight * self._formatRatio < self._formatMaxWidth:
			self._formatWidth = self._formatHeight * self._formatRatio
			self.formatWidth.set_value(round(self._formatWidth))
		else:
			self._formatRatio = self.roundFloat(self._formatWidth / self._formatHeight, 5)
			self.formatRatio.set_value(self._formatRatio)
		self.checkProceed()
		self.setFormatChanged("Height")
					
	def changeRatio(self, widget):
		"""calculates height, width on change of ratio
		and sets the other values"""
		widget.update()
		self.formatHeight.update()
		self.formatWidth.update()
		self._formatRatio = self.roundFloat(widget.get_value(), 5)	
		if self._formatWidth > 0:
			self._formatHeight = round(self._formatWidth / self._formatRatio)
			self.formatHeight.set_value(self._formatHeight)
		else:
			self._formatWidth = round(self._formatHeight * self._formatRatio)
			self.formatWidth.set_value(self._formatWidth)
		self.checkProceed()
		self.setFormatChanged("Ratio")
	
	def changeFramerate(self, widget):
		"""handler for framerate-spin"""
		widget.update()
		self._formatFramerate = self.roundFloat(self.formatFramerate.get_value(), 2)
		self.checkProceed()
		self.setFormatChanged("Framerate")
		
	def changeDeinterlace(self, widget):
		"""handler for deinterlace-button"""
		self._formatDeinterlace = self.formatDeinterlace.get_active()
		
	def changeAudioSamplerate(self, widget):
		"""handler for audio-samplerate"""
		try:
			self._formatAudioSamplerate = eval(self.formatAudioSamplerate.child.get_text())
			self.checkProceed()
			self.setFormatChanged("AudioSamplerate")
		except:
			1
		
	def changeAudioBitrate(self, widget):
		"""handler for audiobitrate"""
		try:
			self._formatAudioBitrate = eval(self.formatAudioBitrate.child.get_text())
			self.calculateTargetInfo()
			self.checkProceed()
			self.setFormatChanged("AudioBitrate")
		except:
			1
		
	def changeForceCodec(self, widget):
		"""handler for audio-codec"""
		model = self.formatForceCodec.get_model()
		iter = self.formatForceCodec.get_active_iter()
		if iter >= 0:
			self._formatForceCodec = model.get_value(iter, 0)
		else:
			self._formatForceCodec = ""
		self.checkProceed()
			
	def changeAudioChannels(self, widget):
		"""handler for audio-channels"""
		try:
			self._formatAudioChannels = eval(self.formatAudioChannels.child.get_text())
			self.checkProceed()
			self.setFormatChanged("AudioChannels")
		except:
			1
		
	def changeDisableAudio(self, widget):
		"""handler for disable-audio"""
		self._formatDisableAudio = self.formatDisableAudio.get_active()
		if self._formatDisableAudio == False and self._hasAudio == False:
			self.messageDialog(self.lang.error_message_noaudio,
				self.lang.error_message_title, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK)
			self._formatDisableAudio = True
			self.formatDisableAudio.set_active(True)
		else:
			self.toggleAudio(self.audioActive())
			self.calculateTargetInfo()
			self.checkProceed()

	def changeDisableVideo(self, widget):
		"""handler for disable-video"""
		self._formatDisableVideo = self.formatDisableVideo.get_active()
		if self._formatDisableVideo == False and self._hasVideo == False:
			self.messageDialog(self.lang.error_message_novideo,
				self.lang.error_message_title, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK)
			self._formatDisableVideo = True
			self.formatDisableVideo.set_active(True)
		else:
			self.toggleVideo(self.videoActive())
			self.calculateTargetInfo()
			self.checkProceed()
	
	def change2Pass(self, widget):
		"""handler for 2-pass encoding switch"""
		self._format2Pass = self.format2Pass.get_active()
			
	def changeCodec(self, widget):
		"""handler for video-codec"""
		model = self.formatCodec.get_model()
		iter = self.formatCodec.get_active_iter()
		if iter >= 0:
			self._formatCodec = model.get_value(iter, 0)
		else:
			self._formatCodec = ""
		self.checkProceed()
			
	def changeBitrate(self, widget, key = False):
		"""handler for bitrate"""
		widget.update()
		self._formatBitrate = self.formatBitrate.get_value()
		self.calculateTargetInfo()
		self.checkProceed()
		self.setFormatChanged("Bitrate")
		
	def changeOffset(self, widget):
		"""handler for offset-value"""
		#widget.update()
		self._formatOffset = self.formatOffset.get_value()
		self.window.get_widget("labelOffsetOut").set_text("(%s)" % self.seconds2timestring(self._formatOffset))
		#self.formatDurationAdjustment.upper = self._inFileDuration - self._formatOffset
		if self._formatDuration > self._inFileDuration + self._inFileOffset - self._formatOffset:
			self._formatDuration = self._inFileDuration + self._inFileOffset - self._formatOffset
			self.formatDuration.set_value(self._formatDuration)
			self.window.get_widget("labelDurationOut").set_text("(%s)" % self.seconds2timestring(self._formatDuration))
		self.calculateTargetInfo()
		self.checkProceed()
		self.setFormatChanged("Offset")
		
	def changeDuration(self, widget):
		"""handler for offset-value"""
		#widget.update()
		self._formatDuration = self.formatDuration.get_value()
		self.window.get_widget("labelDurationOut").set_text("(%s)" % self.seconds2timestring(self._formatDuration))
		#self.formatOffsetAdjustment.upper = self._inFileDuration - self._formatDuration
		if self._formatOffset > self._inFileDuration + self._inFileOffset - self._formatDuration:
			self._formatOffset = self._inFileDuration + self._inFileOffset - self._formatDuration
			self.formatOffset.set_value(self._formatOffset)
			self.window.get_widget("labelOffsetOut").set_text("(%s)" % self.seconds2timestring(self._formatOffset))
		self.calculateTargetInfo()
		self.checkProceed()
		self.setFormatChanged("Duration")
		
	def changeScaling(self, widget):
		"""handler for scalingmethod"""
		self._formatScaling = self.formatScaling.get_active()
		
	def changePadColor(self, widget):
		"""handler for pad-color"""
		color = self.formatPadColor.get_color()
		self._formatPadColor = color.to_string()
		
	def changeFixedRatio(self, widget):
		"""handler for fixed-ratio"""
		self._formatFixedRatio = widget.get_active()
	
	def dragDropInFile(self, widget, source = None, x = None, y = None, data = None, info = None, time = None):
		"""handler for Drag&Drop on inFile-Chooser"""
		uri = data.data.strip()
		if uri[0:7] == "file://":
			self.setInFile(urllib.unquote(uri)[7:])
			self.inFileChanged()
			
	def changeInFile(self, widget):
		"""handler for inFile-Chooser"""
		self.setInFile(self.inFileChooser.get_filename())
		self.inFileChanged()
	
	def changeOutDirectory(self, widget):
		"""handler for the outFileChooser"""
		self._outFile = self.getOutFile()
		self.setOutFile(self._outFile)
		self.analyzeOutFile()
		self.checkProceed()
	
	def changeOutFileSuffix(self, widget):
		"""handler for the suffix-selector in output"""
		self.changeOutFile()
		self._outFile = self.getOutFile()
		self.setOutFile(self._outFile)
		self.analyzeOutFile()
		self.checkProceed()
		
	def changeOutFileName(self, widget, key):
		"""handler for the namefield in output"""
		self.changeOutFile()
		self._outFile = self.getOutFile()
		self.setOutFile(self._outFile)
		self.analyzeOutFile()
		self.checkProceed()
		
	def changeLanguage(self, widget):
		"""handler for languageSelector"""
		model = widget.get_model()
		self._LANGUAGE = model.get_value(widget.get_active_iter(), 0)
		self.setLanguage()
		self.fillLabels()
		if self.inputRead(): self.resetFormatLabels()
		self.fillTextCombos(refill = True)
		self.reloadInFileFilter()
		self.reloadOutFileFilter()
		self.reloadOutFileSuffix()
		self.calculateTargetInfo()

	def triggerReload(self, widget):
		"""Reanalyze input an set the values back"""
		self.changeInFile(widget = False)
		
	def triggerReset(self,widget):
		"""reset all values by clicking the Reset-button"""
		self.reset()	

	def changeToggleMeta(self, widget):
		"""handler for the meta-Enable-switch"""
		self._metaEnable = self.metaToggle.get_active()
		self.toggleMeta(self._metaEnable)
										
#######################################
# SOURCE-OPERATIONS
#######################################	
		
	def setInFile(self, inFile):
		"""check if argument is a file, a directory or nothing
		and set the file and path in the inFileCooser-dialog
		and the _inFile Variable.
		returns true if argument is a file, otherwise false"""
		if inFile != None:
			sfile = os.path.abspath(inFile)
			if os.path.isfile(sfile):
				if self.inFileChooser.get_filename() != sfile:
					self.inFileChooser.unselect_all()
					self.inFileChooser.set_current_folder(os.path.dirname(sfile))
					self.inFileChooser.select_filename(sfile)
				self._inFile = sfile
			elif os.path.isdir(sfile):
				self.inFileChooser.set_current_folder(sfile)
				self.inFileChooser.unselect_all()
				self._inFile = None
			else:
				self.inFileChooser.unselect_all()
				self._inFile = None
		else:
			self.inFileChooser.unselect_all()
			self._inFile = None
		if self._inFile:
			try:
				self.config.add("Settings", "inFile", self._inFile)
			except:
				self.config.set("Settings", "inFile", self._inFile)
			self.config.write(open(self._CFG_SETTINGS, "w"))
		return self._inFile

	def inFileChanged(self):
		"""triggers analyzing on inFile. if file can be read by ffmpeg,
		all GUI-related stuff is executed. otherwise error-dialogs
		are opened. inFile-Info is triggered afterwards"""
		self.analyzeInFile()
		self.displayInFileInfo()
		if self.inputRead():
			self.resetFFmpegOut()
			self.setFormat()
			self.fillFormat()
			self.setMeta()
			self.fillMeta()
			self.setOutFile(self._inFile)
			self.outFileSuffix.set_active(0)
			self.calculateTargetInfo()
		else:
			if self._inFileAccessible == False and self._inFile != None:
				self.messageDialog(self.lang.error_message_file_not_readable,
					self.lang.error_message_title, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK)
			elif self._inFile != None:
				self.messageDialog(self.lang.error_message_file_not_inmedia,
					self.lang.error_message_title, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK)
		self.toggleVideo(self._hasVideo)
		self.toggleAudio(self._hasAudio)
		self.checkProceed()
		self.checkReload()
		

#######################################
# TARGET-OPERATIONS
#######################################	

	def setOutFile(self, outFile):
		"""set the file and path in the outFileChooser-dialog"""
		#print "setOutFile called\nactual:" + self._outFile + "\nrequest:" + outFile
		if outFile != None and outFile != "":
			sfile = os.path.abspath(outFile)
			basename = os.path.basename(sfile)
			dirname = os.path.dirname(sfile)
			if basename != "" and os.path.isdir(sfile) == False:
				if self.outFileSuffix.get_active() > 0:
					outSfxStr = self.getOutFileSuffix()
					if outFile[outFile.rfind("."):] != ("." + outSfxStr) and outSfxStr:
						self.outFileSuffix.set_active(0)
				if self.outFileName.get_text() != basename:
					self.outFileName.set_text(basename)
				if self.outFileChooser.get_current_folder() != dirname:
					self.outFileChooser.set_current_folder(dirname)
				if dirname[:-1] != "/" and dirname != "/":
					self.labelOutDirectory.set_text(dirname + "/")
				else:
					self.labelOutDirectory.set_text(dirname)
				self._outFile = sfile
			elif os.path.isdir(sfile):
				if self.outFileChooser.get_current_folder() != sfile:
					self.outFileChooser.set_current_folder(sfile)
				if self.outFileName.get_text() != "":
					self.outFileName.set_text("")
				if self.outFileSuffix.get_active() != 0:
					self.outFileSuffix.set_active(0)
				if sfile[:-1] != "/" and sfile != "/":
					self.labelOutDirectory.set_text(sfile + "/")
				else:
					self.labelOutDirectory.set_text(sfile)
				self._outFile = sfile
			else:
				self.outFileChooser.set_current_folder(dirname)
				self.outFileName.set_text(basename)
				self.outFileSuffix.set_active(0)
				self.labelOutDirectory.set_text(self._defFile + "/")
				self._outFile = self._defFile
		else:
			self._outFile = None
			
		return self._outFile
		
	def getOutFile(self):
		"""read file and path from outFileChooser-dialog and set outFile"""
		directory = self.outFileChooser.get_current_folder()
		if directory[:-1] != "/" and directory != "/":
			directory += "/"
		self._outFile = os.path.abspath(directory + self.outFileName.get_text())
		return self._outFile	

	def changeOutFile(self):
		outFile = self.outFileName.get_text()
		outSfx = self.outFileSuffix.get_active()
		if outSfx > 0:
			outSfxStr = self.getOutFileSuffix()
			if outFile[outFile.rfind("."):] != ("." + outSfxStr) and outSfxStr:
				if outFile.rfind(".") >= 0:
					outFile = outFile[:outFile.rfind(".")] + "." + outSfxStr
				else:
					outFile += "." + outSfxStr
		self.outFileName.set_text(outFile)

#######################################
# CHECK IF WE CAN GO ON
#######################################			
	
	def checkProceed(self):
		"""checks some values to decide if proceedButton can be enabled"""

		self._canProceed = False
		error = False
		if self._inFileAccessible != True:
			error = self.lang.error_button_not_readable
		elif self._inFileReadable != True:
			error = self.lang.error_button_not_inmedia
		elif self._outFileAccessible != True:
			error = self.lang.error_button_not_writeable
		elif self._outFileWriteable != True:
			error = self.lang.error_button_not_outmedia
		elif self._outFile == self._inFile:
			error = self.lang.error_button_inasout
		elif self._formatDuration == 0:
			error = self.lang.error_button_duration
		elif self._formatWidth == 0 or self._formatHeight == 0:
			error = self.lang.error_button_size
		elif self._formatBitrate == 0 or self._formatAudioBitrate == 0:
			error = self.lang.error_button_bitrate
		elif self._formatAudioSamplerate == 0:
			error = self.lang.error_button_samplerate
		elif self._formatAudioChannels == 0:
			error = self.lang.error_button_channels
		elif self._formatFramerate == 0:
			error = self.lang.error_button_framerate
		elif len(self._formatForceCodec) > 0 and self._formatDisableAudio:
			error = self.lang.error_button_no_audio
		elif self._hasVideo == False and self._formatDisableAudio:
			error = self.lang.error_button_no_audio
		elif self._formatDisableAudio and self._formatDisableVideo:
			error = self.lang.error_button_no_content
		elif len(self._formatForceCodec) > 0 and self._formatAudioBitrate == 0:
			error = self.lang.error_button_no_audio_bitrate
		elif len(self._formatCodec) > 0 and self._formatBitrate == 0:
			error = self.lang.error_button_no_video_bitrate
		else: self._canProceed = True

		if self._canProceed == True:
			label = self.buttonProceedActive()
			self.buttonProceedIcon.set_from_file(sys.path[0] + "/proceed-enabled.svg")
			sens = True
		else:
			label = self.buttonProceedInactive(error)
			self.buttonProceedIcon.set_from_file(sys.path[0] + "/proceed-disabled.svg")
			sens = False
		self.buttonProceedLabel.set_text(label)
		self.buttonProceedIcon.show()
		self.buttonProceed.set_sensitive(sens)
		
	def checkReload(self):
		"""checks some values to decide if ReloadButton can be enabled"""
		self.buttonReload.set_sensitive(self.inputRead())
		
		
#######################################
# CONFIRMATION
#######################################			

	def confirm(self, widget):
		"""IN YOUR FACE!"""
		if self._canProceed:
			
			# build commandline
			options = ""
			# duration
			if self._inFileDuration != self._formatDuration:
				options += " -t " + self.seconds2timestring(self._formatDuration)
			# start
			if self._formatOffset != 0 and self._inFileOffset != self._formatOffset:
				options += " -ss " + self.seconds2timestring(self._formatOffset)
			# video
			if self._hasVideo and self._formatDisableVideo == False:
				if self._inFileBitrate != self._formatBitrate:
					options += " -b %ik" % self._formatBitrate
				if self._inFileFramerate != self._formatFramerate:
					options += " -r %.2f " % self._formatFramerate
				#options += " -bt %i" % round(self._formatBitrate / self._formatBitrateVariation)
				if self._formatCodec != "": options += " -vcodec " + self._formatCodec
				#if self.getOutFileSuffix():
				#	options += " -f %s " % self.getOutFileSuffix()
				# padcolor
				#000000000000
				r = "%02X" % (int(eval("0x" + self._formatPadColor[1:5])) / 256)
				g = "%02X" % (int(eval("0x" + self._formatPadColor[5:9])) / 256)
				b = "%02X" % (int(eval("0x" + self._formatPadColor[9:12])) / 256)
				#r = r[2:].zfill(2)
				#g = g[2:].zfill(2)
				#b = b[2:].zfill(2)
				# pad areas?
				if self._inFileWidth != self._formatWidth or self._inFileHeight != self._formatHeight:
					if self._formatScaling == 0 :
						options += " -padcolor %s%s%s" % (r, g, b)
						if self._inFileRatio < self._formatRatio:
							pad = ((self._formatHeight * self._inFileRatio) - self._formatWidth) / 2
							options += " -padleft %i -padright %i" % self.roundPad(pad)
							options += " -s %ix%i" % self.roundSize(self._formatHeight * self._inFileRatio, self._formatHeight)
						else:
							pad = ((self._formatWidth / self._inFileRatio) - self._formatHeight) / 2
							options += " -padtop %i -padbottom %i" % self.roundPad(pad)
							options += " -s %ix%i" % self.roundSize(self._formatWidth, self._formatWidth / self._inFileRatio)
					
					# or crop areas?
					elif self._formatScaling == 1:
						if self._inFileRatio < self._formatRatio:
							crop = (self._inFileHeight - (self._inFileWidth / self._formatRatio)) / 2
							options += " -croptop %i -cropbottom %i" % (round(crop), round(crop))
						else:
							crop = (self._inFileWidth - (self._inFileHeight * self._formatRatio)) / 2
							options += " -cropleft %i -cropright %i" % (round(crop), round(crop))
						options += " -s %ix%i" % self.roundSize(self._formatWidth, self._formatHeight)
					else:
						options += " -s %ix%i" % self.roundSize(self._formatWidth, self._formatHeight)
			# audio
			if self._formatDisableAudio == False:
				# force codec
				if self._formatForceCodec != "":
					options += " -acodec " + self._formatForceCodec
				#if self._inFileAudioBitrate != self._formatAudioBitrate:
				options += (" -ab %ik" % self._formatAudioBitrate)
				#if self._inFileAudioSamplerate != self._formatAudioSamplerate:
				options += (" -ar %i" % self._formatAudioSamplerate)
				#if self._inFileAudioChannels != self._formatAudioChannels:
				options += (" -ac %i" % self._formatAudioChannels)
				
			elif self._hasAudio:
				options += " -an"
					
			# meta
			if self._metaEnable:
				if self._metaTitle != "": options += " -title '" + self._metaTitle + "'"
				if self._metaAuthor != "": options += " -author '" + self._metaAuthor + "'"
				if self._metaCopyleft != "": options += " -copyright '" + self._metaCopyleft + "'"
				if self._metaComment != "": options += " -comment '" + self._metaComment.encode('string_escape') + "'"
			
			# force overwrite
			if self._inFile == self._outFile or os.path.exists(self._inFile):
				options += " -y"
			# errorhandler
			if self._ERRORHANDLING != False: options += " -er " + self._ERRORHANDLING
			
			# 2 pass
			if self._format2Pass != False: options += " -pass 1"
			
			self._commandline = self._COMMAND % (self._inFile, options, self._outFile)
			
			# show confirmation-dialog
			self.createConfirmation(self.buildConfirmInfo(), self._commandline)
			
					
#######################################
# FINALLY... DO IT!
#######################################	
	def processCall(self, widget):
		"""execute the generated, edited and confirmed command"""
		self.preProcess()
		#gtk.gdk.threads_enter()
		self.ffmpegExec = ffmpegExec(self, self._execute, self._outFile, self._outFileSize, self._formatDuration, self._currpass)
		self.ffmpegExec.start()
		#gtk.gdk.threads_leave()
		
	def proceedCall(self, ffmpegOut, returnCode):
		"""print out the returned lines"""
		self.ffmpegOutText.set_text(ffmpegOut)
		
	def preProcess(self):
		"""some procedures to call before we can beat the monkey"""
		if self._format2Pass == True and self._currentPass == 1:
			# 2-pass - pass 1
			self._execute = self.commandlineText.get_text(self.commandlineText.get_start_iter(), self.commandlineText.get_end_iter(), include_hidden_chars=True)
			self.resetFFmpegOut()
			self.confirmWin.get_widget("tragtorConfirm").destroy()
			self._currpass = 1
		elif self._format2Pass == True:
			# 2-pass - pass 2
			self._execute = self._execute.replace(" -pass 1", " -pass 2")
			self._currpass = 2
		else:
			# single pass
			self._execute = self.commandlineText.get_text(self.commandlineText.get_start_iter(), self.commandlineText.get_end_iter(), include_hidden_chars=True)
			self.resetFFmpegOut()
			self.confirmWin.get_widget("tragtorConfirm").destroy()
			self._currpass = None
			
		self._inProgress = True
		self._cancelProcess = False
		self.buttonProceed.set_sensitive(False)
		
		
	def postProcess(self, returncode, lastline):
		"""clean up when you've finished or trigger 2nd pass"""
		if self._format2Pass == True and self._currentPass == 1:
			self._currentPass = 2
			self.processCall(False)
		else:
			self._currentPass = 1
			self.buttonProceed.set_sensitive(True)
			self.analyzeOutFile()
			self.checkProceed()
			#self.ffmpegOutText.set_text(self._ffmpegSTDOUT)
	
#######################################
# RESET EVERYTHING
#######################################			

	def reset(self, init = False):
		"""Reset all values to default, fills all labels,
		combos and buttons with default values"""
		# reset all internals
		self.resetFormat()
		self.resetMeta()
		# erase ffmpegOut
		self.resetFFmpegOut()
		# reset suffix to "as filename"
		self.outFileSuffix.set_active(0)
		# look for an inFile as argument on startup
		if self._argInFile:
			self.setInFile(self._argInFile)
		else:
			self.setInFile(self._defFile)
				
		if self._argOutFile:
			self.setOutFile(self._argOutFile)
		elif self._argInFile:
			self.setOutFile(self._argInFile)
		else:
			self.setOutFile(self._defFile)
		# analyze whatever
		self.analyzeInFile()
		self.displayInFileInfo()
		# readable file? then adjust format-settings
		if self.inputRead():
			self.setFormat()
			self.setMeta()
		# draw all the labels, combos, values inputs and stuff
		self.fillLabels()
		if self.inputRead(): self.resetFormatLabels()
		self.fillTextCombos(refill = False)
		self.fillFormat()
		self.fillMeta()
		self.calculateTargetInfo()	
		# set the buttons
		self.checkProceed()
		self.checkReload()
		# set format sensitivity
		self.toggleVideo(self._hasVideo)
		self.toggleAudio(self._hasAudio)
		# set meta-sens
		self.toggleMeta(self._metaEnable)
 

#######################################
# ANALYZE SOURCE- AND TARGETFILE
#######################################	

	def analyzeInFile(self):
		"""This operation analyzes a given file with ffmpeg and fills
			inFileFormat[file, format, duration, start, bitrate]
			inFileStreams[n][ID, format, codec, (color or samplerate), (size or channels), (fps or audiobitrate)]
			sets inFileAccessible and inFileReadable also and displays errormesagge if failed
			triggers dependent processes on success"""
		self._hasVideo = False
		self._hasAudio = False
		if self._inFile != "" and self._inFile != None:
			if os.access(self._inFile, os.R_OK) == False or os.path.getsize(self._inFile) <= 1 or os.path.isdir(self._inFile):
				self._inFileAccessible = False
				self._inFileReadable = False
			else:
				self._inFileAccessible = True
				self._inFileReadable = False
				unknown_format = False
				self._inFileStreams = []
				self._inFileFormat = ["", "", "00:00:00.000", "0.0", "N/A"]
				# execute ffmpeg
				ffmpegout = os.popen3("ffmpeg -i '" + self._inFile + "'")
				for line in ffmpegout[2].readlines():
					if self._inStream == 1:
						# parse more formatinfo and append to fileformat:
						# inFileFormat[file, format, duration, start, bitrate]
						print line.strip()
						elements = line.split(", ")
						for el in elements:
							p = el.split(": ")
							if p[0].strip() == "Duration":
								self._inFileFormat[2] = p[1].strip()
							elif p[0].strip() == "start":
								self._inFileFormat[3] = p[1].strip()
							elif p[0].strip() == "bitrate":
								self._inFileFormat[4] = p[1].strip()
								if self._inFileFormat[4] != "N/A": self._inFileFormat[4] = self._inFileFormat[4][:-5]
						self._inStream = 2;
					elif self._inStream > 1:
						# parse streams inside Input
						# inFileStreams[n][ID, format, codec, (video or samplerate), (size or channels), (fps or audiobitrate)]
						if not re.search("Must supply at least one output file", line):
							print line.strip()
						stream = re.search("Stream #([0-9]+\.[0-9]+)(\[0x[0-9A-Za-z]+\])?: (Audio|Video): ([A-Za-z0-9\-_]+),", line)
						if stream:
							array = [stream.group(1), stream.group(3), stream.group(4), "0", "0", "0"]
							if array[1] == "Audio":
								substr = line.strip()[len(stream.group(0)):]
								sr = re.search("([0-9]+) Hz", substr)
								if sr:
									array[3] = sr.group(1)
								ch = re.search("(stereo|mono),", substr)
								if ch:
									array[4] = ch.group(1)
								br = re.search("([0-9]+) kb/s", substr)
								if br:
									array[5] = br.group(1)
								self._hasAudio = True
								self._inFileStreams.append(array)
								
							elif array[1] == "Video":
								substr = line.strip()[len(stream.group(0)):]
								me = re.search("\A(([0-9A-Za-z])+),", substr.strip())
								if me:
									array[3] = me.group(1)
								sz = re.search("(([0-9]+)x([0-9]+))[ \,]{1}", substr)
								if sz:
									array[4] = sz.group(1)
								fr = re.search("([0-9]{1,2}(\.[0-9]{0,2}){0,1}) (fps|tb)(\([cr]\)){0,1}", substr)
								if fr:
									array[5] = fr.group(1)
								self._hasVideo = True	
								self._inFileStreams.append(array)
								
							self._inStream += 1
						else:
							self._inStream = 0;

					elif line[:5] == "Input":
						print line.strip()
						# Begin Input and set fileformat:
						# inFileFormat[file, format]
						elements = line.split(", ")
						self._inFileFormat[0] = elements[2][6:-3]
						self._inFileFormat[1] = elements[1]
						self._inStream = 1;
						
					if line[-15:] == "Unknown format":
						unknown_format = True
						
				if unknown_format == False and len(self._inFileStreams) > 0:
					self._inFileReadable = True

				self._inFileFormat.append(os.path.getsize(self._inFile))
				ffmpegout[0].close()
			
				return self._inFileReadable
		else:
			self._inFileAccessible = False
			return False

	def analyzeOutFile(self):
		"""analyzes, if selected outfile is writeable"""
		#self.getOutFile()
		if self._outFile != None:
			if os.access(os.path.dirname(os.path.abspath(self._outFile)), os.W_OK) == False:
				self._outFileAccessible = False
			else:
				self._outFileAccessible = True
			filename = os.path.basename(self._outFile)
			suffix = filename[filename.rfind(".") + 1:]
			for s in self.ffmpegConfig._saveFileFormats:
				if s[0].lower() == suffix.lower() and s[0] != "":
					self._outFileWriteable = True
					self._outFileFormat = s
					break
			return (self._outFileAccessible, self._outFileWriteable)
		else:
			return False
		
#######################################
# SET INTERNAL VALUES
#######################################				
	
	def setFormat(self):
		"""Fill the internal format-values with presets depending on sourcefile"""
		
		# Video-Audio-options
		#inFileStreams[n][ID, format, codec, (color or samplerate), (size or channels), (fps or audiobitrate)]
			
		for stream in self._inFileStreams:
			if stream[1] == "Video":
				xy = stream[4].split("x")
				self._formatWidth = self._inFileWidth = int(xy[0])
				self._formatHeight = self._inFileHeight = int(xy[1])
				self._formatRatio = self._inFileRatio = self.roundFloat(float(self._formatWidth) / float(self._formatHeight), 5)
				self._formatFramerate = self._inFileFramerate = eval(stream[5])
				#self._formatCodec = stream[2]
				if self._inFileFormat[4] != "N/A":
					self._formatBitrate = self._inFileBitrate = int(self._inFileFormat[4])
				else:
					self._formatBitrate = self._inFileBitrate = 0
			if stream[1] == "Audio":
				self._formatAudioSamplerate = self._inFileAudioSamplerate = eval(stream[3])
				self._formatAudioBitrate = self.calulateNewBitrate(eval(stream[5]))
				self._inFileAudioBitrate = eval(stream[5])
				
				if stream[4] == "mono":
					self._formatAudioChannels =	1
				else:
					self._formatAudioChannels =	2
				self._inFileAudioChannels = self._formatAudioChannels
		# Duration
		self._formatDuration = self._inFileDuration = self.timestring2seconds(self._inFileFormat[2])
		# Offset
		if self._formatFramerate <= 0.0: self._formatFramerate = self._defaultFramerate
		fs = self.seconds2frames(eval(self._inFileFormat[3]), self._formatFramerate)
		s = int(fs[2] % 60)
		m = int(fs[2] / 60)
		h = int(m / 60)
		self._formatOffset = self._inFileOffset = self.timetuple2seconds((h, m, s, fs[3]))
			
	def setMeta(self):
		"""Fill the meta-related internal values depending on sourcefile"""
		self._metaComment = self.lang.meta_comment_text
		self._metaTitle = os.path.basename(self._inFile)[:os.path.basename(self._inFile).rfind(".")].title()
		self._metaAuthor = ""
		self._metaCopyright = ""
		id3v2obj = id3v2(self._inFile)
		if id3v2obj.id3v1:
			if id3v2obj.id3v1["Title"] != "":
				self._metaTitle = id3v2obj.id3v1["Title"]
			if id3v2obj.id3v1["Artist"] != "":
				self._metaAuthor = id3v2obj.id3v1["Artist"]
			if id3v2obj.id3v1["Comment"] != "":
				self._metaComment = id3v2obj.id3v1["Comment"]
				

	def setLanguage(self):
		"""changes the language-object to internal language"""
		self.lang = eval("lang_" + self._LANGUAGE + ".tragtorLang")()
		try:
			self.config.add("Settings", "language", self._LANGUAGE)
		except:
			self.config.set("Settings", "language", self._LANGUAGE)
		self.config.write(open(self._CFG_SETTINGS, "w"))
		
#######################################
# RESET INTERNAL VALUES
#######################################
	
	def resetFormat(self):
		"""resets the internal values to default"""
		self._formatWidth = self._defaultWidth
		self._formatHeight = self._defaultHeight
		self._formatRatio = self._defaultRatio
		self._formatFixedRatio = self._defaultFixedRatio
		self._formatFramerate = self._defaultFramerate
		self._formatDeinterlace = self._defaultDeinterlace
		self._formatBitrate = self._defaultBitrate
		self._formatAudioSamplerate = self._defaultAudioSamplerate
		self._formatAudioBitrate = self._defaultAudioBitrate
		self._formatForceCodec = self._defaultForceCodec
		self._formatAudioChannels = self._defaultAudioChannels
		self._formatDisableAudio = self._defaultDisableAudio
		self._formatCodec = self._defaultCodec
		self._formatDisableVideo = self._defaultDisableVideo
		self._formatOffset = self._defaultOffset
		self._formatDuration = self._defaultDuration
		self._formatScaling = self._defaultScaling
		self._formatPadColor = self._defaultPadColor
		self._formatHiQ = self._defaultHiQ
		self._formatBitrateVariation = self._defaultBitrateVariation
		self._format2Pass = self._default2Pass
		
	def resetMeta(self):
		"""resets internal metaData to none"""
		self._metaTitle = ""
		self._metaAuthor = ""
		self._metaCopyleft = ""
		self._metaComment = ""
		self._metaEnable = self._defaultMetaEnable

#######################################
# SET INTERNAL VALUES DEPENDING ON PRESET
#######################################

	def presetFormat(self, name):
		for i in self._DEFAULT_PRESET:
			value = self.presets.get(name, i)
			if re.search(r"[A-Za-z\-_#]", value) and not re.match(r"(True|False)", value) or value == "":
					value = "'%s'" % value
			exec "self._format%s = %s" % (i, value)

#######################################
# FILL THE GUI WITH VALUES AND LABELS
#######################################

	def fillFormat(self):
		"""fill the format-widgets with object-values"""
		if self.inputRead():
			self.formatDurationAdjustment.upper = self._inFileDuration + self._inFileOffset
			self.formatOffsetAdjustment.upper = self._inFileDuration + self._inFileOffset
		else:
			self.formatDurationAdjustment.upper = 0
			self.formatOffsetAdjustment.upper = 0
			
		
		self.formatOffset.set_value(self._formatOffset)
		self.formatDuration.set_value(self._formatDuration)
		self.formatRatio.set_value(self._formatRatio)
		self.formatWidth.set_value(self._formatWidth)
		self.formatHeight.set_value(self._formatHeight)
		
		self.formatFramerate.set_value(self._formatFramerate)
		self.formatBitrate.set_value(self._formatBitrate)
		self.formatAudioSamplerate.child.set_text("%i" % self._formatAudioSamplerate)
		self.formatAudioBitrate.child.set_text("%i" % self._formatAudioBitrate)
		self.formatAudioChannels.child.set_text("%i" % self._formatAudioChannels)
		if self._formatForceCodec in self.ffmpegConfig._saveAudioCodecs:
			self.formatForceCodec.set_active(self.ffmpegConfig._saveAudioCodecs.index(self._formatForceCodec) + 1)
		else:
			self.formatForceCodec.set_active(-1)
		if self._formatCodec in self.ffmpegConfig._saveVideoCodecs:
			self.formatCodec.set_active(self.ffmpegConfig._saveVideoCodecs.index(self._formatCodec) + 1)
		else:
			self.formatCodec.set_active(-1)
			
		self.formatScaling.set_active(self._formatScaling)
		self.formatFixedRatio.set_active(self._formatFixedRatio)
		self.format2Pass.set_active(self._format2Pass)
		self.formatPadColor.set_color(gtk.gdk.color_parse(self._formatPadColor))
		
	def fillMeta(self):
		self.metaTitle.set_text(self._metaTitle)
		self.metaAuthor.set_text(self._metaAuthor)
		self.metaCopyleft.set_text(self._metaCopyleft)
		self.metaComment.set_text(self._metaComment)
				
	def fillLabels(self):
		"""Fill al the labels with language-depending text"""
		self.window.get_widget("labelInput").set_text(self.lang.infile_label)
		self.window.get_widget("labelOutput").set_text(self.lang.outfile_label)
		self.window.get_widget("labelSuffix").set_text(self.lang.suffix_label)
		self.window.get_widget("labelFormatAudio").set_text(self.lang.format_audio_label)
		self.window.get_widget("labelFormatVideo").set_text(self.lang.format_video_label)
		self.window.get_widget("labelFormatRender").set_text(self.lang.format_render_label)
		self.window.get_widget("labelFormatResults").set_text(self.lang.format_results_label)
		self.window.get_widget("labelMeta").set_text(self.lang.meta_label)
		self.window.get_widget("labelProceed").set_text(self.lang.proceed_label)
		self.window.get_widget("labelSettings").set_text(self.lang.settings_label)
		
		self.window.get_widget("notebookInput").set_text(self.lang.infile_notebook)
		self.window.get_widget("notebookOutput").set_text(self.lang.outfile_notebook)
		self.window.get_widget("notebookFormat").set_text(self.lang.format_notebook)
		self.window.get_widget("notebookMeta").set_text(self.lang.meta_notebook)
		self.window.get_widget("notebookProceed").set_text(self.lang.proceed_notebook)
		self.window.get_widget("notebookSettings").set_text(self.lang.settings_notebook)
		
		self.window.get_widget("labelFormatWidth").set_text(self.lang.format_width_label)
		self.window.get_widget("labelFormatHeight").set_text(self.lang.format_height_label)
		self.window.get_widget("labelFormatRatio").set_text(self.lang.format_ratio_label)
		self.window.get_widget("labelFormatFixed").set_text(self.lang.format_fixedratio_label)
		self.window.get_widget("labelFormatFramerate").set_text(self.lang.format_framerate_label)
		self.window.get_widget("labelFormatDeinterlace").set_text(self.lang.format_deinterlace_label)
		
		self.window.get_widget("labelFormatAudioSamplerate").set_text(self.lang.format_audiosamplerate_label)
		self.window.get_widget("labelFormatAudioBitrate").set_text(self.lang.format_audiobitrate_label)
		self.window.get_widget("labelFormatForceCodec").set_text(self.lang.format_forcecodec_label)
		self.window.get_widget("labelFormatAudioChannels").set_text(self.lang.format_audiochannels_label)
		self.window.get_widget("labelFormatDisableAudio").set_text(self.lang.format_disableaudio_label)
		
		self.window.get_widget("labelFormatCodec").set_text(self.lang.format_codec_label)
		self.window.get_widget("labelFormatBitrate").set_text(self.lang.format_bitrate_label)
		self.window.get_widget("labelFormatOffset").set_text(self.lang.format_offset_label)
		self.window.get_widget("labelFormatDuration").set_text(self.lang.format_duration_label)
		self.window.get_widget("labelFormatScaling").set_text(self.lang.format_scaling_label)
		self.window.get_widget("labelFormatPadColor").set_text(self.lang.format_padcolor_label)
		self.window.get_widget("labelFormatDisableVideo").set_text(self.lang.format_disablevideo_label)
		self.window.get_widget("labelFormat2Pass").set_text(self.lang.format_2pass_label)
		
		self.window.get_widget("labelPresets").set_text(self.lang.presets_label)
		
		self.window.get_widget("labelMetaTitle").set_text(self.lang.meta_title_label)
		self.window.get_widget("labelMetaAuthor").set_text(self.lang.meta_author_label)
		self.window.get_widget("labelMetaCopyleft").set_text(self.lang.meta_copyleft_label)
		self.window.get_widget("labelMetaComment").set_text(self.lang.meta_comment_label)	
		self.window.get_widget("labelMetaEnable").set_text(self.lang.meta_toggle_label)
		
		self.window.get_widget("labelSettingsLanguage").set_text(self.lang.settings_language_label)	
		self.window.get_widget("labelSettingsReload").set_text(self.lang.settings_reload_label)	
		self.window.get_widget("labelSettingsReset").set_text(self.lang.settings_reset_label)	
		self.window.get_widget("buttonLabelSettingsReload").set_text(self.lang.settings_reload_button)	
		self.window.get_widget("buttonLabelSettingsReset").set_text(self.lang.settings_reset_button)	
		
		self.window.get_widget("labelSettingsCredits").set_text(self.lang.credits % (self._TITLE, self._VERSION, self._AUTHOR, self.lang.translation_by))
		
		self.checkProceed()
	
	def fillTextCombos(self, refill):
		"""fills all configured comboBoxes with language-specific values"""
		for i in range(len(self._languageCombos)):
			array = eval("self.lang." + self._languageCombos[i][1])
			obj = eval("self." + self._languageCombos[i][0])
			val = eval("self._" + self._languageCombos[i][0])
			lst = obj.get_model()
			if refill:
				lst.clear()
			for c in range(len(array)):
				obj.append_text(array[c])
			obj.set_active(val)

	#inFile Filters
	def reloadInFileFilter(self):
		"""reloads the language-specific entries in the
		fileselector on source-page"""
		if self.inFileChooser.get_filter() != None:
			self.inFileChooser.remove_filter(self.inFileFilter)
			self.inFileFilter.destroy()
		self.inFileFilter = gtk.FileFilter()
		self.inFileFilter.set_name(self.lang.title_infile_mediafiles)
		for f in self.ffmpegConfig._openFileFormats:
			self.inFileFilter.add_pattern("*" + f[0])
		self.inFileChooser.add_filter(self.inFileFilter)

		self.inFileNoFilter = gtk.FileFilter()
		self.inFileNoFilter.set_name(self.lang.title_infile_allfiles)
		self.inFileNoFilter.add_pattern("*")
		self.inFileChooser.add_filter(self.inFileNoFilter)
	
	#outFile Filters
	def reloadOutFileFilter(self):
		"""reloads the language-specific entries in the
		fileselector-dialog on target-page"""
		if self.outFileChooser.get_filter() != None:
			self.outFileChooser.remove_filter(self.outFileFilter)
			self.outFileFilter.destroy()
		self.outFileFilter = gtk.FileFilter()
		self.outFileFilter.set_name(self.lang.title_outfile_mediafiles)
		for f in self.ffmpegConfig._saveFileFormats:
			self.outFileFilter.add_pattern("*" + f[0])
		self.outFileChooser.add_filter(self.outFileFilter)

		self.outFileNoFilter = gtk.FileFilter()
		self.outFileNoFilter.set_name(self.lang.title_outfile_allfiles)
		self.outFileNoFilter.add_pattern("*")
		self.outFileChooser.add_filter(self.outFileNoFilter)
		
	def reloadOutFileSuffix(self):
		"""reloads the language-specific part in the
		suffix-selector on target-page"""
		active = self.outFileSuffix.get_active()
		self.outFileSuffix.remove_text(0)
		self.outFileSuffix.insert_text(0, self.lang.output_file_suffix_auto)
		self.outFileSuffix.set_active(active)

#######################################
# FORMAT AND DISPLAY TEXT
#######################################

	def displayInFileInfo(self):
		"""format and display the informations about inFile on first page"""
		output = ""
		if self.inputRead():
			output += self.lang.infile_format_info % (
				self._inFileFormat[0],
				self.bytes2string(self._inFileFormat[5]),
				self._inFileFormat[1],
				self._inFileFormat[4],
				self._inFileFormat[2])
			for stream in self._inFileStreams:
				st = self.lang.infile_stream_main_info
				if stream[1] == "Video":
					st += self.lang.infile_stream_video_info
				elif stream[1] == "Audio":
					st += self.lang.infile_stream_audio_info
				output += st % (stream[0], stream[1], stream[2], stream[3], stream[4], stream[5])		
			self.inFileText.set_text(self.lang.infile_textarea_pre + output + self.lang.infile_textarea_post) 
		elif self._inFileAccessible:
			self.inFileText.set_text(self.lang.error_infile_not_readable % (self._inFile, self._compileOptions,
				" ".join(["%s" % (k) for k, v in self.ffmpegConfig._openFileFormats]), " ".join(["%s" % (k) for k, v in self.ffmpegConfig._saveFileFormats]),
				" ".join(self.ffmpegConfig._openAudioCodecs), " ".join(self.ffmpegConfig._saveAudioCodecs),
				" ".join(self.ffmpegConfig._openVideoCodecs), " ".join(self.ffmpegConfig._saveVideoCodecs),
				))
		elif self._inFile != None:
			self.inFileText.set_text(self.lang.error_infile_not_accessible % self._inFile)
		else:
			self.resetInFileInfo()			
	
	def resetInFileInfo(self):
		"""resets the inFile-Info-Textarea to blank"""
		self.inFileText.set_text("")
		
	def resetFFmpegOut(self):
		"""resets the ffmpegOut-Textarea to blank"""
		self.ffmpegOutText.set_text("")
		
	def calculateTargetInfo(self):
		"""generates a readable string with output-results"""
		if self.inputRead():
			videosize = self._formatDuration * self._formatBitrate
			audiosize = self._formatDuration * self._formatAudioBitrate
			overall = self.kbs2string(videosize + audiosize)
			audiosize = self.kbs2string(audiosize)
			videosize = self.kbs2string(videosize)
			if self._formatDisableAudio and self._formatDisableVideo == False:
				output = self.lang.format_target_info % (
					self.lang.format_target_info_video % (self.seconds2human(self._formatDuration), videosize))
			elif self._formatDisableAudio == False and self._formatDisableVideo == False:
				output = self.lang.format_target_info % (
					self.lang.format_target_info_av % (self.seconds2human(self._formatDuration), overall))
			elif self._formatDisableAudio == False and self._formatDisableVideo:
				output = self.lang.format_target_info % (
					self.lang.format_target_info_audio % (self.seconds2human(self._formatDuration), audiosize))
			else:
				output = self.lang.format_target_info_nothing
			self.formatResultInfo.set_markup(output)
		else:
			self.formatResultInfo.set_markup(self.lang.format_target_nofile)

	def buildConfirmInfo(self):
		"""creates the infotext for the confirmation-dialog from lang-files"""
		videosize = self._formatDuration * self._formatBitrate
		audiosize = self._formatDuration * self._formatAudioBitrate
		output = (self.lang.label_confirm_infile % self._inFile) + "\n"
		output += (self.lang.label_confirm_insize % self.bytes2string(self._inFileFormat[5])) + "\n\n"
		output += (self.lang.label_confirm_outfile % self._outFile) + "\n"
		if self._formatDisableAudio and self._formatDisableVideo == False:
			output += self.lang.label_confirm_outsize % self.kbs2string(videosize) + "\n\n"
			self._outFileSize = videosize
		elif self._formatDisableAudio == False and self._formatDisableVideo == False:
			output += self.lang.label_confirm_outsize % self.kbs2string(videosize + audiosize) + "\n\n"
			self._outFileSize = videosize + audiosize
		elif self._formatDisableAudio == False and self._formatDisableVideo:
			output += self.lang.label_confirm_outsize % self.kbs2string(audiosize) + "\n\n"
			self._outFileSize = audiosize
		else:
			output +=  "\n"
		output += self.lang.label_confirm_format % (self._outFileFormat[0] + " (" + self._outFileFormat[1]) + ")\n"
		if self._hasVideo and self._formatDisableVideo != True:
			output += self.lang.label_confirm_outfile % ("%ix%i" % (self._formatWidth, self._formatHeight)) + "\n"
			if self._formatCodec != "":
				output += self.lang.label_confirm_video % (self._formatCodec) + "\n"
		if self._hasAudio and self._formatDisableAudio != True:
			if self._formatForceCodec != "":
				output += (self.lang.label_confirm_audio % self._formatForceCodec) + "\n"
		if self._formatDisableAudio:
			output += (self.lang.label_confirm_audio % self.lang.label_confirm_audiodisabled) + "\n"
		output += self.lang.label_confirm_offset % self.seconds2human(self._formatOffset) + "\n"
		output += self.lang.label_confirm_duration % self.seconds2human(self._formatDuration) + "\n"
		return output
				

#######################################
# HELPERS AND STUFF
#######################################

	def roundFloat(self, value, digits):
		value *= 10 ** digits
		return round(value) / 10 ** digits

	def roundSize(self, width, height):
		width = round(width)
		height = round(height)
		if width % 2 == 1: width += 1
		if height % 2 == 1: height += 1
		return (width, height)

	def roundPad(self, pad):
		if pad < 0 : pad *= -1
		if pad % 1:
			pad1 = int(pad)
			pad2 = int(pad) + 1
		else: pad1 = pad2 = pad
		if pad1 % 2 == 1: pad1 -= 1
		if pad2 % 2 == 1: pad2 += 1
		return (pad1, pad2)
	
	def getOutFileSuffix(self):
		outSfx = self.outFileSuffix.get_active()
		if outSfx > 0:
			model = self.outFileSuffix.get_model()
			iter = self.outFileSuffix.get_active_iter()
			if iter > 0:
				return model.get_value(iter, 0)
			else:
				return False
		else:
			return False

	def seconds2frames(self, seconds, fps):
		"""converts a given second-float to frames and seconds, no zero-fps allowed"""
		ms = int(seconds * 1000)
		fms = 1000 / fps
		frames = int(ms / fms)
		framesover = int(frames % fps)
		seconds = int(frames / fps)
		msover = framesover * fms
		return [frames, framesover, seconds, msover]

	def seconds2timetuple(self, seconds):
		if seconds > 0:
			h = int(seconds / (60 * 60))
			m = int((seconds - (h * 60 * 60)) / 60)
			s = int(seconds - (h * 60 * 60) - (m * 60))
			x = int(round((seconds % 1) * 1000))
			return (h, m, s, x)
		else:
			return (0, 0, 0, 0)
	
	def seconds2timestring(self, seconds):
		"""generates hh:mm:ss.xxx from seconds"""
		return "%.2d:%.2d:%.2d.%.3d" % self.seconds2timetuple(seconds)
			
	def seconds2human(self, seconds):
		"""converts a timetuple in a readable string"""
		return self.timetuple2human(self.seconds2timetuple(seconds))

	def timetuple2seconds(self, timetuple):
		"""returns a timetuple to seconds-float"""
		h, m, s, x = timetuple
		return float((h * 3600) + (m * 60) + s + (float(x) / 1000))

	def timetuple2timestring(self, timetuple):
		"""creates a string mhhh:mmm:sss:xxx from tuple"""
		h, m, s, x = timetuple
		return "%.2d:%.2d:%.2d.%.3d" % (h, m, s, x)	

	def timetuple2human(self, timetuple):
		"""converts a timetuple in a readable string"""
		h, m, s, x = timetuple
		out = ""
		if h > 0 and h == 1: out += ("%i " % h) + self.lang.hour + ", "
		elif h > 0: out += ("%i " % h) + self.lang.hours + ", "
		if m > 0 and m == 1: out += ("%i " % m) + self.lang.minute + " "
		elif m > 0: out += ("%i " % m) + self.lang.minutes + " "
		elif len(out) > 0: out = out[:-2] + " "
		if len(out) > 0 and s > 0: out += self.lang._and + " "
		if s > 0 and s == 1: out += ("%i " % s) + self.lang.second
		elif s > 0 or len(out) == 0: out += ("%i " % s) + self.lang.seconds
		return out
		
	def timestring2timetuple(self, string):
		"""creates a valid timetuple from string"""
		ints = []
		p = re.compile('[0-9]+')
		items = p.findall(string)
		while len(items) < 4:
			items.append("0")
		for i in items:
			ints.append(int(i))
		return tuple(ints)
		
	def timestring2seconds(self, string):
		"""creates a valid second-float from string"""
		return self.timetuple2seconds(self.timestring2timetuple(string))
		
	def bits2bytes(self, bits):
		"""returns bytes calculated from bits"""
		return round(bits / 8)
		
	def kbits2bytes(self, bits):
		"""returns bytes calculated from bits"""
		return round(bits / 8 / 1024)	
			
	def bytes2string(self, value):
		"""returns a formatted string from bytes (1024)"""
		value = float(value)
		if value >= (1024**4) :
			return self.scaleFloat(value/(1024.0**4)) + " T"
		if value >= (1024**3) :
			return self.scaleFloat(value/(1024.0**3)) + " G"
		if value >= (1024**2) :
			return self.scaleFloat(value/(1024.0**2)) + " M"
		if value >= 1024 :
			return self.scaleFloat(value/1024.0) + " k"
		else : return ("%i" % value) + ""

	def kbytes2string(self, value):
		"""returns a formatted string from kbytes (1024)"""
		value = float(value)
		if value >= (1024**4) :
			return self.scaleFloat(value/(1024.0**4)) + " P"
		if value >= (1024**3) :
			return self.scaleFloat(value/(1024.0**3)) + " T"
		if value >= (1024**2) :
			return self.scaleFloat(value/(1024.0**2)) + " G"
		if value >= 1024 :
			return self.scaleFloat(value/1024.0) + " M"
		else : return ("%i" % value) + " k"

	def kbs2string(self, value):
		"""returns a formatted string from kbits(1000)"""
		value = float(value) / 8
		if value >= (1024**4) :
			return self.scaleFloat(value/(1000.0**4)) + " P"
		if value >= (1024**3) :
			return self.scaleFloat(value/(1000.0**3)) + " T"
		if value >= (1024**2) :
			return self.scaleFloat(value/(1000.0**2)) + " G"
		if value >= 1024 :
			return self.scaleFloat(value/1000.0) + " M"
		else : return ("%i" % value) + " k"

	def scaleFloat(self, value):
		"""scales floatings to readable sizes"""
		if value >= 100 :
			return "%d" % round(value)
		if value >= 10 :
			return "%.1f" % value
		else : return "%.2f" % value
	
	def inputRead(self):
		"""returns if inFile is read- and accessible"""
		if self._inFileReadable and self._inFileAccessible:
			return True
		else:
			return False
			
	def outputWrite(self):
		"""returns if outFile is write- and accessible"""
		if self._outFileWriteable and self._outFileAccessible:
			return True
		else:
			return False
			
	def videoActive(self):
		"""returns true, if formatVDisableVideo is False"""
		if self._formatDisableVideo: return False
		else: return True
		
	def audioActive(self):
		"""returns true, if formatVDisableVideo is False"""
		if self._formatDisableAudio: return False
		else: return True

	def buttonProceedInactive(self, error):
		return self.lang.proceed_button_inactive % error
		
	def buttonProceedActive(self):
		if self._inFile == self._outFile or os.path.exists(self._outFile):
			return self.lang.proceed_button_overwrite
		else:
			return self.lang.proceed_button_active
			
	def calulateNewBitrate(self, bitrate):
		if bitrate < 8: bitrate = self._DEFAULTBITRATE
		while True:
			try:
				i = self._AUDIOBITRATES.index(bitrate)
				break
			except ValueError:
				bitrate -= 1
				i = -1
		if i >= 0: return self._AUDIOBITRATES[i]
		else: return bitrate
	
	def setFormatChanged(self, object):
		try:
			var = eval("self._inFile" + object)
		except(AttributeError):
			pass
		else:
			if self.window.get_widget("button" + object):
				if self.inputRead():
					if eval("self._inFile" + object) != eval("self._format" + object):
						self.window.get_widget("button" + object).show()
					else:
						self.window.get_widget("button" + object).hide()
				else: self.window.get_widget("button" + object).hide()
			
	def resetFormatLabels(self):
		for label in self._CACHEFORMAT:
			self.setFormatChanged(label)		
		
#######################################
# TOGGLE AUDIO/VIDEO SENSITIVITY
#######################################	
		
	def toggleVideo(self, active):
		self.imageFormatVideo.set_sensitive(active)
		self.window.get_widget("labelFormatVideo").set_sensitive(active)
		self.window.get_widget("labelFormatCodec").set_sensitive(active)
		self.formatCodec.set_sensitive(active)
		self.window.get_widget("labelFormatBitrate").set_sensitive(active)
		self.formatBitrate.set_sensitive(active)
		self.window.get_widget("labelFormatFramerate").set_sensitive(active)
		self.formatFramerate.set_sensitive(active)
		self.window.get_widget("labelFormatWidth").set_sensitive(active)
		self.formatWidth.set_sensitive(active)
		self.window.get_widget("labelFormatHeight").set_sensitive(active)
		self.formatHeight.set_sensitive(active)
		self.window.get_widget("labelFormatRatio").set_sensitive(active)
		self.formatRatio.set_sensitive(active)
		self.window.get_widget("labelFormatFixed").set_sensitive(active)
		self.formatFixedRatio.set_sensitive(active)
		self.window.get_widget("labelFormatDeinterlace").set_sensitive(active)
		self.formatDeinterlace.set_sensitive(active)
		self.window.get_widget("labelFormatScaling").set_sensitive(active)
		self.formatScaling.set_sensitive(active)
		self.window.get_widget("labelFormatPadColor").set_sensitive(active)
		self.formatPadColor.set_sensitive(active)
		if self.formatDisableVideo.get_active() and active:
			self.formatDisableVideo.set_active(False)
		if self.formatDisableVideo.get_active() == False and active == False:
			self.formatDisableVideo.set_active(True)
		
	def toggleAudio(self, active):
		self.imageFormatAudio.set_sensitive(active)
		self.window.get_widget("labelFormatAudio").set_sensitive(active)
		self.window.get_widget("labelFormatForceCodec").set_sensitive(active)
		self.formatForceCodec.set_sensitive(active)
		self.window.get_widget("labelFormatAudioBitrate").set_sensitive(active)
		self.formatAudioBitrate.set_sensitive(active)
		self.window.get_widget("labelFormatAudioSamplerate").set_sensitive(active)
		self.formatAudioSamplerate.set_sensitive(active)
		self.window.get_widget("labelFormatAudioChannels").set_sensitive(active)
		self.formatAudioChannels.set_sensitive(active)
		if self.formatDisableAudio.get_active() and active:
			self.formatDisableAudio.set_active(False)
		if self.formatDisableAudio.get_active() == False and active == False:
			self.formatDisableAudio.set_active(True)

	def toggleMeta(self, active):
		self.window.get_widget("labelMetaTitle").set_sensitive(active)
		self.window.get_widget("labelMetaAuthor").set_sensitive(active)
		self.window.get_widget("labelMetaCopyleft").set_sensitive(active)
		self.window.get_widget("labelMetaComment").set_sensitive(active)
		self.metaTitle.set_sensitive(active)
		self.metaAuthor.set_sensitive(active)
		self.metaCopyleft.set_sensitive(active)
		self.metaComment.set_sensitive(active)
		if self.metaToggle.get_active() == False and active:
			self.metaToggle.set_active(True)
		if self.metaToggle.get_active() and active == False:
			self.metaToggle.set_active(False)
						
#######################################
# WINDOW-FUNCTIONS
#######################################

	def messageDialog(self, message, title = "tragtor-Message", behavior = gtk.MESSAGE_INFO, buttons = gtk.BUTTONS_OK, image = None):
		"""creates a message-dialog with variable tile, message, behavior and buttons"""
		dialog = gtk.MessageDialog(
			parent=None,
			flags=gtk.DIALOG_DESTROY_WITH_PARENT,
			type=behavior,
			buttons=buttons,
			message_format=message)
		dialog.set_title(title)
		if image:
			dialog.set_image = image
			dialog.set_icon = image
		dialogreturn = dialog.run()
		dialog.destroy()
		return dialogreturn
		
	def createConfirmation(self, labeltext, commandline):
		self.confirmWin = gtk.glade.XML(self.gladefile, "tragtorConfirm",)
		self.commandlineText = gtk.TextBuffer()
		self.confirmWin.get_widget("textCommandline").set_buffer(self.commandlineText)
		self.commandlineText.set_text(commandline)
		self.confirmWin.get_widget("tragtorConfirm").set_title(self.lang.title_confirmation_window)
		self.confirmWin.get_widget("labelConfirmation").set_text(self.lang.label_confirmation)
		self.confirmWin.get_widget("labelCommandline").set_text(self.lang.label_confirm_commandline)
		self.confirmWin.get_widget("labelConfirmationInfo").set_text(labeltext)
		handlers = {
			"on_buttonConfirmOK_clicked" : self.processCall,
			"on_buttonConfirmCancel_clicked" : self.killConfirm,
			}
		self.confirmWin.signal_autoconnect(handlers)
		return self.confirmWin

	def killConfirm(self, widget):
		self.confirmWin.get_widget("tragtorConfirm").destroy()

	def createWindow(self):
		"""create the main-window"""
		self.gladefile = sys.path[0] + "/tragtor.glade"  
		self.window = gtk.glade.XML(self.gladefile, "tragtor","tragtor")
		
	def destroy(self, widget):
		"""wtf?!1"""
		gtk.main_quit()

	def ctrlC(self, signum, frame):
		self.destroy(None)

	def createWidgetObjects(self):
		"""here we wrap all widgets to local objects for easier calls"""
		self.inFileChooser = self.window.get_widget("inFileChooser")
		self.outFileChooser = self.window.get_widget("outFileChooser")
		self.labelOutDirectory = self.window.get_widget("labelOutDirectory")
		self.outFileName = self.window.get_widget("outFileName")
		self.outFileSuffix = self.window.get_widget("outFileSuffix")
		self.languageSelector = self.window.get_widget("languageSelector")
		self.inFileInfo = self.window.get_widget("inFileInfo")
		self.buttonProceed = self.window.get_widget("buttonProceed")
		self.buttonProceedIcon =self.window.get_widget("buttonProceedIcon")
		self.buttonProceedLabel = self.window.get_widget("buttonProceedLabel")
		self.ffmpegOut = self.window.get_widget("ffmpegOut")
		
		self.buttonReload = self.window.get_widget("buttonReload")
		self.buttonReset = self.window.get_widget("buttonReset")
		
		self.formatWidth = self.window.get_widget("formatWidth")
		self.formatHeight = self.window.get_widget("formatHeight")
		self.formatRatio = self.window.get_widget("formatRatio")
		self.formatFixedRatio = self.window.get_widget("formatFixedRatio")
		self.formatFramerate = self.window.get_widget("formatFramerate")
		self.formatDeinterlace = self.window.get_widget("formatDeinterlace")
		self.formatAudioSamplerate = self.window.get_widget("formatAudioSamplerate")
		self.formatAudioBitrate = self.window.get_widget("formatAudioBitrate")
		self.formatForceCodec = self.window.get_widget("formatForceCodec")
		self.formatAudioChannels = self.window.get_widget("formatAudioChannels")
		self.formatDisableAudio = self.window.get_widget("formatDisableAudio")
		self.formatCodec = self.window.get_widget("formatCodec")
		self.formatDisableVideo = self.window.get_widget("formatDisableVideo")
		self.format2Pass = self.window.get_widget("format2Pass")
		self.formatBitrate = self.window.get_widget("formatBitrate")
		self.formatOffset = self.window.get_widget("formatOffset")
		self.formatOffsetOut = self.window.get_widget("formatOffsetOut")
		self.formatDuration = self.window.get_widget("formatDuration")
		self.formatDurationOut = self.window.get_widget("formatDurationOut")
		self.formatScaling = self.window.get_widget("formatScaling")
		self.formatPadColor = self.window.get_widget("formatPadColor")
		self.formatResultInfo = self.window.get_widget("formatResultInfo")
		self.imageFormatVideo = self.window.get_widget("imageFormatVideo")
		self.imageFormatAudio = self.window.get_widget("imageFormatAudio")
		self.imageFormatRender = self.window.get_widget("imageFormatRender")
		self.imageFormatResults = self.window.get_widget("imageFormatResults")
		
		self.metaTitle = self.window.get_widget("metaTitle")
		self.metaAuthor = self.window.get_widget("metaAuthor")
		self.metaCopyleft = self.window.get_widget("metaCopyleft")
		self.metaComment = self.window.get_widget("metaComment")
		self.metaToggle = self.window.get_widget("metaToggle")
		
		self.presetName = self.window.get_widget("presetName")
		self.presetSave = self.window.get_widget("presetSave")
		self.presetOpen = self.window.get_widget("presetOpen")
		self.presetDelete = self.window.get_widget("presetDelete")
	
	def start_config(self, cfgfile, defaults={}, sections=[], optionxform_off = False):
		if not os.path.exists(os.path.dirname(cfgfile)):
			os.mkdir(os.path.dirname(cfgfile))
		if not os.path.exists(cfgfile):
			c = open(cfgfile, "w")
			c.close()
		cfg = config.RawConfigParser(defaults)
		if optionxform_off:
			cfg.optionxform = str
		cfg.readfp(open(cfgfile))
		for s in sections:
			if not cfg.has_section(s):
				cfg.add_section(s)
		return cfg
	
			
	def troubleShooter(self, widget, ee):
		"""This function represents the truth!"""
		self._eec += 1
		if self._eec >=8:
			self.eeWin = gtk.glade.XML(self.gladefile, "tragtorEE",)
			self._eec = 0	
	
class ffmpegConfig():
	"""this class reads the config of ffmpeg and stores it in various lists"""
	_ffmpegVersion = ""
	_compileOptions = ""
	_openFileFormats = []
	_saveFileFormats = []
	_openAudioCodecs = []
	_openVideoCodecs = []
	_saveAudioCodecs = []
	_saveVideoCodecs = []
	def  __init__(self):
		"""parse the config from the comandline and fill
			compileOptions
			openAudioCodecs
			openVideoCodecs
			saveAudioCodecs
			saveVideoCodecs
			openFileFormats
			saveFileFormats"""
		inFormats = False
		inCodecs = False
		ffmpegout = os.popen3("ffmpeg -formats")
		for line in ffmpegout[2].readlines():
			line = line.strip()
			if line[:7] == "FFmpeg ":
				self._ffmpegVersion = line
			if line[:14] == "configuration:":
				self._compileOptions = line[16:].replace(" ", "\n")
		for line in ffmpegout[1].readlines():
			if inFormats:
				if line[1:2] == "D":
					pos = string.find(line[4:-1],' ')
					format = line[4:pos+5].strip()
					desc = line[pos+5:-1].strip()
					self._openFileFormats.append([format, desc])
				if line[2:3] == "E":
					pos = string.find(line[4:-1],' ')
					format = line[4:pos+5].strip()
					desc = line[pos+5:-1].strip()
					self._saveFileFormats.append([format, desc])
			elif inCodecs:
				if line[1:2] == "D":
					if line[3:4] == "A":
						self._openAudioCodecs.append(line[8:-1])
					if line[3:4] == "V":
						self._openVideoCodecs.append(line[8:-1])
				if line[2:3] == "E":
					if line[3:4] == "A":
						self._saveAudioCodecs.append(line[8:-1])
					if line[3:4] == "V":
						self._saveVideoCodecs.append(line[8:-1])
			if line[:13] == "File formats:":
				inFormats = True
			if line[:7] == "Codecs:":
				inFormats = False
				inCodecs = True		
		ffmpegout[0].close()
			
class ffmpegExec(Thread):
	"""This class is the ffmpeg-executable"""

	_execute = ""
	_caller = None
	_ffmpegOut = ""
	
	_running = False
	_stop = False
	
	def  __init__(self, caller, command, outfile, outfilesize, outfileduration, currpass = None):
		"""init to given command"""
		self._stop = False
		self._running = False
		self._execute = command
		self._caller = caller
		self._file = outfile
		self._filesize = outfilesize
		self._fileduration = outfileduration
		self._currpass = currpass
		Thread.__init__(self)

	def run(self):
		"""execute the ffmpeg-command and return the output."""
		
		timeline = re.compile(".*size=.* time=.* bitrate=")
		
		self.progressWin = self.createProgressWin()
		
		self._running = True
		
		self._PROCESS = sub.Popen(self._execute,
			stdout = sub.PIPE,
			stderr = sub.STDOUT,
			stdin = sub.PIPE,
			universal_newlines = True,
			shell = True)
		lastline = ""
		while self._PROCESS.poll() == None and self._stop == False:
			out = self._PROCESS.stdout.readline().strip()
			if out != "Press [q] to stop encoding":
				if timeline.match(out) == None and out != "":
					self._ffmpegOut += out + "\n"
					gobject.idle_add(self._caller.proceedCall, self._ffmpegOut, self._PROCESS.returncode)
				else:
					gobject.idle_add(self.updateProgressText, out)	
			gobject.idle_add(self.updateProgressBar, out)
		self._PROCESS.stdout.close()
		self._running = False
		if self.progressWin.get_widget("tragtorProgress") != None:
			self.progressWin.get_widget("progressProgress").set_fraction(1)
			time.sleep(0.5)
			self.progressWin.get_widget("tragtorProgress").destroy()
		self._caller.postProcess(self._PROCESS.returncode, self._ffmpegOut.splitlines().pop())
		return False
							
	def stop(self, widget):
		"""Stop method, sets the event to terminate the thread's main loop"""
		if self._running:
			self._PROCESS.stdin.write("q")
			self._PROCESS.stdin.close()
		self._stop = True
		return False
		
	def createProgressWin(self):
		self.progressWin = gtk.glade.XML(self._caller.gladefile, "tragtorProgress",)
		self.progressWin.get_widget("labelProgress").set_text(self._caller.lang.label_progress)
		if self._currpass != None:
			title = "%s %s" % (self._caller.lang.title_progress_window, self._caller.lang.title_progress_window_pass % self._currpass)
		else:
			title = self._caller.lang.title_progress_window
		self.progressWin.get_widget("tragtorProgress").set_title(title)

		handlers = {
			"on_buttonProgress_clicked" : self.stop,
			"destroy" : self.stop,
			"delete_event" : self.stop
			}
		self.progressWin.signal_autoconnect(handlers)
		return self.progressWin
		
	def updateProgressText(self, line):
		if self.progressWin.get_widget("text1Progress") != None:
			txt1 = ""
			txt2 = ""
			c = re.search(r"size=[\s]*([0-9]+(\.[0-9]+)?)", line)
			if c:
				txt1 = "%s\n%s" % (txt1, self._caller.lang.label_progress_size)
				txt2 = "%s\n%sBytes" % (txt2, self._caller.kbytes2string(eval(c.group(1))))
			c = re.search(r"time=[\s]*([0-9]+(\.[0-9]+)?)", line)
			if c:
				txt1 = "%s\n%s" % (txt1, self._caller.lang.label_progress_time)
				txt2 = "%s\n%s s" % (txt2, c.group(1))
			c = re.search(r"bitrate=[\s]*([0-9]+(\.[0-9]+)?)", line)
			if c:
				txt1 = "%s\n%s" % (txt1, self._caller.lang.label_progress_bitrate)
				txt2 = "%s\n%s kB/s" % (txt2, c.group(1))
			c = re.search(r"frame=[\s]*([0-9]+(\.[0-9]+)?)", line)
			if c:
				txt1 = "%s\n%s" % (txt1, self._caller.lang.label_progress_frame)
				txt2 = "%s\n%s" % (txt2, c.group(1))
			c = re.search(r"q=[\s]*([0-9]+(\.[0-9]+)?)", line)
			if c:
				txt1 = "%s\n%s" % (txt1, self._caller.lang.label_progress_quantizer)
				txt2 = "%s\n%s" % (txt2, c.group(1))
			txt1 = txt1.strip()
			txt2 = txt2.strip()
			self.progressWin.get_widget("text1Progress").set_markup(txt1)
			self.progressWin.get_widget("text2Progress").set_markup(txt2)
		else:
			return False
	
	def updateProgressBar(self, line):
		duration = re.search(r"time=([0-9]+\.[0-9]+)", line)
		if duration:
			dur = eval(duration.group(1))
			progress = dur / self._fileduration
			if progress > 1.0 : progress = 1
			if self.progressWin.get_widget("progressProgress") != None:
				self.progressWin.get_widget("progressProgress").set_fraction(progress)
			else:
				return False

class id3v2():
	"""this one interacts with the commandline-tool id3v2.
	at the moment it is able to read id3v1-tags, in the future
	a complete wrapper with read/write-support for v1 and v2 
	is planned"""
	
	emptyVal = ""
	
	id3v1 = False
	id3v2 = False

	def __init__(self, file):
		"""analyze the file"""
		self.id3v1 = False
		self.id3v2 = False
		if os.access(file, os.R_OK):
			id3v2out = os.popen3("id3v2 -l " + file)
			inV1 = False
			inV2 = False
			countV1 = 0
			for line in id3v2out[1].readlines():
				line = line.strip()
				if inV1:
					if countV1 == 0:
						if line[9:39].strip() != "":
							self.id3v1 = {"Title": line[9:39].strip()}
						else:
							self.id3v1 = {"Title": self.emptyVal}
						if line[49:].strip() != "":
							self.id3v1["Artist"] = line[49:].strip()
						else:
							self.id3v1["Artist"] = self.emptyVal
					if countV1 == 1:
						if line[9:39].strip() != "":
							self.id3v1["Album"] = line[9:39].strip()
						else:
							self.id3v1["Album"] = self.emptyVal
						if line[47:51].strip() != "":
							self.id3v1["Year"] = line[47:51].strip()
						else:
							self.id3v1["Year"] = self.emptyVal
						if line[60:line.rfind("(")].strip() != "":
							self.id3v1["Genre"] = line[60:line.rfind("(")].strip()
						else:
							self.id3v1["Genre"] = self.emptyVal
						if line[(line.rfind("(") + 1):line.rfind(")")].strip() != "":
							self.id3v1["GenreNum"] = line[(line.rfind("(") + 1):line.rfind(")")].strip()
						else:
							self.id3v1["GenreNum"] = self.emptyVal
					if countV1 == 2:
						if line[9:39].strip() != "":
							self.id3v1["Comment"] = line[9:39].strip()
						else:
							self.id3v1["Comment"] = self.emptyVal
						if line[48:].strip() != "":
							self.id3v1["Track"] = line[48:].strip()
						else:
							self.id3v1["Track"] = self.emptyVal
					countV1 += 1

				if inV2:
					self.id3v2 = True
				if line == "id3v1 tag info for %s:" % file:
					inV1 = True
				if line == "id3v2 tag info for %s:" % file:
					inV2 = True
					inV1 = False
			id3v2out[0].close()
		else:
			return False
		
def main_quit(obj):
	"""main_quit function, it stops the thread and the gtk's main loop"""
	gtk.main_quit()
			
#######################################
# EVERYTHING HAS TO COME TO AN END
#######################################			
			
if __name__ =="__main__":
	tragtorWindow = tragtorGUI()
	gtk.main()

